const fs = require("fs");
const fss = require("fs-extra");
const mkdirp = require('mkdirp');
const path = require('path');
var file_conf_name = null;
var finfos = null;
const fileInfo = null;
var cfg_path = null;
const lockFile = [];
var document = null;
const lockType = {
	remote: 0,
	fileInfos: 1,
}
const lockFd = [null, null];

function load(_path, _fname, _doc) {
	if (_path) cfg_path = _path;
	if (_fname) file_conf_name = cfg_path + '/' + _fname;
	if (_doc) document = _doc;
	mkdirp.sync(cfg_path);
	if (!fs.existsSync(file_conf_name)) {
		finfos = {
			files: {},
			path: null,
		};
	} else {
		var fjson = "";
		var bufjson = fs.readFileSync(file_conf_name, 'utf8');
		finfos = JSON.parse(bufjson.toString());
	}

	if (!lockFile[lockType.remote]) lockFile[lockType.remote] = cfg_path + '/' + '.remote.lock';
	if (!lockFile[lockType.fileInfos]) lockFile[lockType.fileInfos] = cfg_path + '/' + '.finfos.lock';
	if (fs.existsSync(lockFile[lockType.remote])) fs.unlink(lockFile[lockType.remote]);
	if (fs.existsSync(lockFile[lockType.fileInfos])) fs.unlink(lockFile[lockType.fileInfos]);
}

function save(cb) {
	waitLock(lockType.fileInfos, function () {
		console.log("Saving config file");
		fs.writeFileSync(file_conf_name, JSON.stringify(finfos, null, 4));
		if (cb) cb();
	});
	var ev = new Event('update_sensors');
	document.dispatchEvent(ev);
	/*
	if (!save.hdle) save.hdle = [];
	var hdle = setInterval(function () {
		if (getLock(lockType.fileInfos)) {
			fs.writeFileSync(fname, JSON.stringify(finfos, null, 4));
			unLock(lockType.fileInfos);
			clearInterval(save.hdle.shift());
		}
	}, 1000);
	save.hdle.push(hdle);
	*/
}

function addFile(fname) {
	finfos.files[fname] = {
		offset: 0,
		nb_items_recv: 0,
		nb_items_sent: 0,
	};
}
function getOrSetFilePath(path) {
	if (!finfos.path && path) finfos.path = path;
	return finfos.path;
}
function updateFilesvalue(fname, vals) {
	if (!finfos.files[fname]) addFile(fname);
	if (vals) {
		if (vals.offset) finfos.files[fname].offset = vals.offset;
		if (vals.nb_items_recv) finfos.files[fname].nb_items_recv = vals.nb_items_recv;
	}
}

function incFilesValues(fname, vals) {
	if (!finfos.files[fname]) addFile(fname);
	if (vals) {
		if (vals.offset) finfos.files[fname].offset += vals.offset;
		if (vals.nb_items_recv) finfos.files[fname].nb_items_recv += vals.nb_items_recv;
		if (vals.nb_items_sent) finfos.files[fname].nb_items_sent += vals.nb_items_sent;
	}
}

function getFiles() {
	return Object.keys(finfos.files);
}

function getValue(fname, key) {
	return finfos.files[fname][key];
}

function getValues(fname, key) {
	var f = fname.split('.')[0].split('_');
	return {
		serial: f[1],
		dau_id: f[3],
		stype: f[4],
		sid:f[5],
		nb_items_recv: getValue(fname, 'nb_items_recv'),
		nb_items_sent: getValue(fname, 'nb_items_sent'),
	};
}

function getSensorStats(cb) {
	if (!cb) return;
	var files = getFiles();
	files.forEach(function (item) {
		cb(getValues(item));
	});
}

function fileLock() {
	return getLock(lockType.remote);
}

function fileUnlock() {
	return unLock(lockType.remote);
}

function fileWaitLock(timeout, cb) {
	waitLock(lockType.remote, cb, timeout);
}

function waitLock(ltype, cb, timeout) {
	if (!waitLock.hdle) {
		waitLock.hdle = [[], []];
	}

	var hdle = setInterval(function () {
		if (getLock(ltype)) {
			if (cb) cb();
			unLock(ltype);
			clearInterval(waitLock.hdle[ltype].shift());
		}
	}, timeout ? timeout : 1000);
	waitLock.hdle[ltype].push(hdle);
}

function getLock(lType) {
	var ret = false;

	try {
		lockFd[lType] = fs.openSync(lockFile[lType], "wx");
		ret = true;
	} catch (err) {
		if (err.code == 'EEXIST')
			lockFd[lType] = null;
		else {
			throw err;
		}
	}
	return ret;
}

function unLock(lType) {
	if (!lockFd[lType]) return;
	fs.closeSync(lockFd[lType]);
	if (fs.existsSync(lockFile[lType])) fs.unlinkSync(lockFile[lType]);
	lockFd[lType] = null;
}

function checkAllSent() {
	var allFiles = getFiles();
	all_sent = 1;
	allFiles.forEach(function (fname) {
		if (getValue(fname, 'nb_items_sent') < getValue(fname, 'nb_items_recv')) {
			all_sent = 0;
			return false;
		}
	});
	return all_sent;
}

function removeFile(fname) {
	fs.unlinkSync(fname);
}

function removeAll() {
	fss.removeSync(cfg_path);
	load();
}
module.exports = {
	load: load,
	save: save,
	getOrSetFilePath: getOrSetFilePath,
	addFile: addFile,
	incFilesValues: incFilesValues,
	getFiles: getFiles,
	getValue: getValue,
	getValues: getValues,
	fileLock: fileLock,
	fileUnlock: fileUnlock,
	fileWaitLock: fileWaitLock,
	getSensorStats: getSensorStats,
	checkAllSent: checkAllSent,
	removeFile: removeFile,
	removeAll: removeAll
};
